package com.seeyon.ctp.portal.sso;

import com.eetrust.security.plugin.MessageConstants;
import com.eetrust.security.plugin.SIDPlugin;
import com.seeyon.ctp.common.SystemEnvironment;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.util.StringUtils;

import java.io.*;
import java.util.Properties;

public class MySSOLoginHandshake extends SSOLoginHandshakeAbstract {
    private static final Log log = LogFactory.getLog(MySSOLoginHandshake.class);
    // “ticket” 就是ticket取得的参数值
    public String handshake(String ticket) {
        if (StringUtils.isEmpty(ticket))
        {
            log.error( "单点登录缺少登录票据[ticket]信息!");
            return null;
        }
        // 认证服务器地址
        String authServerIp = "";

        // 认证服务器端口
        String authServerPort = "";

        // 业务系统编码
        String authAppCode = "";

        String filePath = SystemEnvironment.getApplicationFolder() + File.separator + "fkmap" + File.separator + "url.properties" ;
        InputStream in = null;
        try {
            in = new BufferedInputStream(new FileInputStream(filePath));
            Properties p = new Properties();
            p.load(in);
            authServerIp = p.getProperty("authServerIp");
            authServerPort = p.getProperty("authServerPort");
            authAppCode = p.getProperty("authAppCode");
        } catch (Exception e) {
            e.printStackTrace();
        }

        SIDPlugin sid = new SIDPlugin(authServerIp, Integer.parseInt(authServerPort));
        int result = sid.Security_VerifyTicket(authAppCode, ticket);
//        log.info("ticket:"+ticket);
        if (result == MessageConstants.SECURITY_SERVICE_SUCCESS)
        {
            // 票据验证成功, 获取用户唯一标识
            String passport = sid.getPassport();
            log.info("系统获取到的ticket:"+ticket);
            log.info("单点登录成功! 用户唯一标识: " + passport);
            return passport;
        }
        else
        {
            // 认证失败, 获取错误编码
            String errorCode = sid.getErrorCode();

            // 认证失败, 获取错误信息
            String errorMsg = sid.getErrorMsg();
            log.error( "单点登录失败! 错误编码: " + errorCode + ", 错误信息: " + errorMsg);
            return null;
        }
    }
    public void logoutNotify(String ticket) {
        log.info("退出登录："+ticket);
    }

}